@@ -95,20 +95,20 @@ use vars qw{
   @site_rules_path
 };
 
-$VERSION = "3.001005";      # update after release (same format as perl $])
+$VERSION = "3.001006";      # update after release (same format as perl $])
 #$IS_DEVEL_BUILD = 1;        # change for release versions
 
 @ISA = qw();
 
 # SUB_VERSION is now just <yyyy>-<mm>-<dd>
-$SUB_VERSION = (split(/\s+/,'$LastChangedDate: 2006-08-29 08:18:34 -0700 (Tue, 29 Aug 2006) $ updated by SVN'))[1];
+$SUB_VERSION = (split(/\s+/,'$LastChangedDate: 2006-10-03 21:56:19 -0700 (Tue, 03 Oct 2006) $ updated by SVN'))[1];
 
 # If you hacked up your SA, you should add a version_tag to your .cf files.
 # This variable should not be modified directly.
 @EXTRA_VERSION = qw();
 if (defined $IS_DEVEL_BUILD && $IS_DEVEL_BUILD) {
   push(@EXTRA_VERSION,
-       ('r' . qw{$LastChangedRevision: 438093 $ updated by SVN}[1]));
+       ('r' . qw{$LastChangedRevision: 452727 $ updated by SVN}[1]));
 }
 
 sub Version {
@@ -1278,6 +1278,10 @@ sub lint_rules {
 
   $self->{dont_copy_prefs} = $olddcp;       # revert back to previous
 
+  # bug 5048: override settings to ensure a faster lint
+  $self->{'conf'}->{'use_auto_whitelist'} = 0;
+  $self->{'conf'}->{'bayes_auto_learn'} = 0;
+
   my $mail = $self->parse(\@testmsg, 1);
   my $status = Mail::SpamAssassin::PerMsgStatus->new($self, $mail,
                         { disable_auto_learning => 1 } );
